from sys import platform
from importlib import import_module


def notify(title, message, **kwargs):
    for os in ['linux', 'win32', 'darwin']:
        if platform.startswith(os):
            module = import_module('ntfy.backends.{}'.format(os))
            module.notify(title=title, message=message, **kwargs)
            break



import dbus
from os import path

DEFAULT_ICON = path.join(path.split(path.split(__file__)[0])[0], 'icon.png')

def notify(title, message, icon=DEFAULT_ICON, **kwargs):
    bus = dbus.SessionBus()
    dbus_obj = bus.get_object(
        'org.freedesktop.Notifications', '/org/freedesktop/Notifications')
    dbus_iface = dbus.Interface(
        dbus_obj, dbus_interface='org.freedesktop.Notifications')

    dbus_iface.Notify('ntfy', 0, path.abspath(icon), title, message, [], {}, -1)



# -- coding: utf-8 --

from win32api import *
from win32gui import *
import win32con
import os
import struct
import time


DEFAULT_ICON = os.path.join(os.path.split(
    os.path.split(__file__)[0])[0], 'icon.ico')


def notify(title, message, icon=DEFAULT_ICON, **kwargs):
    class WindowsBalloonTip:
        def __init__(self, title, msg):
            message_map = {
                    win32con.WM_DESTROY: self.OnDestroy,
            }
            # Register the Window class.
            wc = WNDCLASS()
            hinst = wc.hInstance = GetModuleHandle(None)
            wc.lpszClassName = "PythonTaskbar"
            wc.lpfnWndProc = message_map # could also specify a wndproc.
            classAtom = RegisterClass(wc)
            # Create the Window.
            style = win32con.WS_OVERLAPPED | win32con.WS_SYSMENU
            self.hwnd = CreateWindow( classAtom, "Taskbar", style, \
                    0, 0, win32con.CW_USEDEFAULT, win32con.CW_USEDEFAULT, \
                    0, 0, hinst, None)
            UpdateWindow(self.hwnd)
            iconPathName = os.path.abspath(icon)
            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
            try:
                hicon = LoadImage(hinst, iconPathName, \
                        win32con.IMAGE_ICON, 0, 0, icon_flags)
            except:
                hicon = LoadIcon(0, win32con.IDI_APPLICATION)
            flags = NIF_ICON | NIF_MESSAGE | NIF_TIP
            nid = (self.hwnd, 0, flags, win32con.WM_USER+20, hicon, "tooltip")
            Shell_NotifyIcon(NIM_ADD, nid)
            Shell_NotifyIcon(NIM_MODIFY, \
                            (self.hwnd, 0, NIF_INFO, win32con.WM_USER+20,\
                            hicon, "Balloon  tooltip",title,200,msg))
            DestroyWindow(self.hwnd)
        def OnDestroy(self, hwnd, msg, wparam, lparam):
            nid = (self.hwnd, 0)
            Shell_NotifyIcon(NIM_DELETE, nid)
            PostQuitMessage(0) # Terminate the app.


    WindowsBalloonTip(message, title)



import requests


def notify(title, message, access_token, device_iden=None, email=None, **kwargs):
    """
    Required config keys:
        * 'access_token'
    """

    data = {
        'type': 'note',
        'title': title,
        'body': message,
    }
    if device_iden is not None:
        data['device_iden'] = device_iden
    if email is not None:
        data['email'] = email

    headers = {'Access-Token': access_token}

    resp = requests.post('https://api.pushbullet.com/v2/pushes',
                         data=data, headers=headers)

    resp.raise_for_status()



import requests


def notify(title, message, user_key,
           api_token='aUnsraBiEZVsmrG89AZp47K3S2dX2a', device=None, **kwargs):
    """
    Required config keys:
        * 'user_key'
    """

    data = {
        'message': message,
        'token': api_token,
        'user': user_key,
        'title': title,
    }
    if device:
        data['device'] = device

    resp = requests.post('https://api.pushover.net/1/messages.json',
                         data=data)

    resp.raise_for_status()



import Foundation
import objc
import AppKit

NSUserNotification = objc.lookUpClass('NSUserNotification')
NSUserNotificationCenter = objc.lookUpClass('NSUserNotificationCenter')

def notify(title, message, **kwargs):
    """
    adapted from https://gist.github.com/baliw/4020619
    """
    notification = NSUserNotification.alloc().init()
    notification.setTitle_(title)
    if message is not None:
        notification.setInformativeText_(message)
    notification.setDeliveryDate_(Foundation.NSDate.date())

    NSUserNotificationCenter.defaultUserNotificationCenter()\
        .scheduleNotification_(notification)



import json
from sys import stderr
from os.path import expanduser


def load_config(config_path='~/.ntfy.json'):
    try:
        config = json.load(open(expanduser(config_path)))
    except IOError:
        stderr.write(
            "Warning: Couldn't open config file '{}'.\n".format(config_path))
        config = {'backends': ['default']}

    if 'backend' in config:
        if 'backends' in config:
            stderr.write("Warning: both 'backend' and 'backends' in config, "
                         "ignoring 'backend'.\n")
        else:
            config['backends'] = [config['backend']]

    return config



from setuptools import setup
from sys import platform

deps = ['requests', 'sleekxmpp']

if platform == 'win32':
    deps.append('pypiwin32')

setup(
    name='ntfy',

    version='0.5.0',

    description='A utility for sending push notifications',

    url='https://github.com/dschep/ntfy',

    author='Daniel Schep',
    author_email='dschep@gmail.com',

    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',

        'Environment :: Console',

        'Intended Audience :: End Users/Desktop',

        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='push notification',

    packages=['ntfy', 'ntfy.backends'],
    package_data={'ntfy': ['icon.png', 'icon.ico']},

    install_requires=deps,

    entry_points={
        'console_scripts': [
            'ntfy = ntfy.cli:main',
        ],
    },
)



import argparse
import json
from os.path import expanduser
from getpass import getuser
from importlib import import_module
from socket import gethostname
from subprocess import call
from sys import stderr, exit
from time import time

from requests import HTTPError


def load_config(args):
    try:
        config = json.load(open(expanduser(args.config)))
    except IOError:
        stderr.write(
            "Warning: Couldn't open config file '{.config}'.\n".format(args))
        config = {'backends': ['default']}

    if 'backend' in config:
        if 'backends' in config:
            stderr.write("Warning: both 'backend' and 'backends' in config, "
                         "ignoring 'backend'.\n")
        else:
            config['backends'] = [config['backend']]

    if args.backend:
        config['backends'] = args.backend

    return config


def run_cmd(args):
    start_time = time()
    retcode = call(args.command)
    return '"{}" {} in {:d}:{:02d} minutes'.format(
        ' '.join(args.command),
        'succeeded' if retcode == 0 else 'failed',
        *map(int, divmod(time() - start_time, 60))
    )


def send_notification(message, args, config):
    ret = 0

    for backend in config['backends']:
        module = import_module('ntfy.backends.{}'.format(backend))

        try:
            backend_config = config.get(backend, {})
            backend_config.update(args.option)
            module.notify(title=args.title, message=message,
                          **backend_config)
        except HTTPError as e:
            stderr.write(
                'Error: status={resp.status_code} body={resp.content}\n'.format(
                    resp=e.response))
            ret = 1

    return ret


def main():
    parser = argparse.ArgumentParser(
        description='Send push notification when command finishes')

    parser.add_argument('-c', '--config',
                        default='~/.ntfy.json',
                        help='config file to use (default: ~/.ntfy.json)')
    parser.add_argument('-b', '--backend', action='append',
                        help='override backend specified in config')
    parser.add_argument('-o', '--option', nargs=2, action='append',
                        metavar=('key', 'value'), default=[],
                        help='backend specific options')

    default_title = '{}@{}'.format(getuser(), gethostname())

    parser.add_argument('-t', '--title', default=default_title,
                        help='a title for the notification (default: {})'
                        .format(default_title))
    parser.add_argument('-d', '--device', help='device to notify')

    subparsers = parser.add_subparsers()

    send_parser = subparsers.add_parser('send', help='send a notification')
    send_parser.add_argument('message',
                             help='notification message')
    send_parser.set_defaults(func=lambda args: args.message)

    done_parser = subparsers.add_parser(
        'done', help='run a command and send a notification when done')
    done_parser.add_argument('command',
                             nargs=argparse.REMAINDER,
                             help='command to run')
    done_parser.set_defaults(func=run_cmd)

    args = parser.parse_args()
    config = load_config(args)

    if hasattr(args, 'func'):
        return send_notification(args.func(args), args, config)
    else:
        parser.print_help()

if __name__ == '__main__':
    exit(main())
